/*global jQuery, window, document, alert */

(function($) {

	$.param.traditional = true;

	function triggerReady( self ) {
		if ( !self.data('atb-init') ) {
			self.data('atb-init', new Date())
				.trigger('atb-init');
		}
		
		self.trigger('atb-refresh')
			.attr('aria-hidden', false);
		
		// Ensure that the atb-ready event is triggered after the display
		// has been refreshed.
		window.setTimeout(function() {
			if ( self.is(':focusable') ) {
				self.focus();
			}
			self.trigger('atb-ready');
		}, 1);
	}
	
	function loadTemplate( self, path ) {
		self.trigger('atb-loading');

		$.ajax({
			url: path + ".action?decorator=none",
			success: function(html) {
				self.html(html);
				triggerReady(self);
			},
			error: function() {
				alert('Failed to load template');
				
				if ( window.console && window.console.log ) {
					window.console.log(arguments);
				}
				
				self.attr('aria-invalid', true)
					.trigger('atb-error');
			},
			complete: function() {
				self.trigger('atb-complete');
			}
		});
	}

// Load templates
$(':role[data-template]')
	.roleStage('bind', function() {
		$(this)
			.bind('pre-attr.@aria-hidden', function(event) {
				if ( !$.dt.bool(event.newValue) ) {
					var target = $(event.target);
			
					// Prevent the element from becoming visible until loaded
					event.preventDefault();
				
					target
						.unbind('pre-attr', arguments.callee)
						.trigger('atb-activate');
				}
			});
	});

// Sidebar toggle buttons
$('.ui-column-toggle-left')
	.roleStage('bind', function() {
		$(this)
			.bind('click', function() {
				$(this).closest('.ui-column-container').toggleClass('ui-column-collapse-left');
				return false;
			});
	});

// Make all fieldsets collapsible - TODO: Implement this as an ARIA widget
//$('legend')
//	.roleStage('bind', function() {
//		$(this)
//			.bind('click', function() {
//				$(this).siblings().toggle('normal');
//			});
//	});

// Setup activation controls
$('[data-activate]')
	.roleStage('bind', function() {
		$(this)
			.bind('click', function() {
				var targets = $.attr(this, 'data-activate').split(/\s+/);
			
				(function() {
					if ( targets.length ) {
						$('#' + targets.shift())
							.one('atb-ready', arguments.callee)
							.trigger('atb-activate');
					}
				})();
			});
	});

$(document)
	// The activate event will cause an element to be loaded (if necessary), refreshed and displayed.
	.bind('atb-activate', function(event) {
		var target = $(event.target),
			template = target.attr('data-template'),
			loaded = target.data('atb-init');
		
		if ( template && !loaded ) {
			loadTemplate(target, template);
		} else {
			// Use a timeout for consistency with loadTemplate
			// where triggerReady would be a callback
			window.setTimeout(function() {
				triggerReady(target);
			}, 0);
		}
	})
	
	// Setup newly added sections of HTML
	.bind('html', function(event) {
	
		// Move head only elements into head, otherwise Safari ignores them
		$('link,meta,style', event.target).appendTo('head');
		
		// Setup roles
		$(event.target).roleSetup();
	});

})(jQuery);

