/*! Build-0.3: datatypes.core.js datatypes.utils.js datatypes.attr.js datatypes.idrefs.js */
/*!
 * jQuery Datatypes 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
/* Datatype parsing functions.
 */
(jQuery.dt || (function($) {

$.dt = {
	string:
		function( str ) { return str === null || str === undefined ? undefined : ''+str; },
		
	bool:
		function( str ) {
			return str === true || str === 'true' || str === '1' ? true :
				str === false || str === 'false' || str === '0' ? false :
				undefined;
		},
		
	number:
		parseFloat,
	
	idref:
		function( str ) {
			return $( str ? document.getElementById(str) : [] );
		},
	
	idrefs:
		function( str ) {
			str = $.trim(str);
			return $( str ? '#' + str.split(/\s+/).join(',#') : [] );
		},
	
	tokens:
		function( str ) {
			return $( str ? str.split(/\s+/) : [] );
		},
	
	integer:
		function( str ) {
			return parseInt(str, 10);
		}
};

})(jQuery)
);

/*
 * jQuery Datatypes Utility Functions 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
/* Optional dependencies:
 *   datatypes.core.js
 *   datatypes.xsd.js
 *   datatypes.idrefs.js
 */
(function($) {

var fns = {
	// Convert an object/array/jQuery to a raw string
	write:
		function( value ) {
			if ( value === undefined || value === null ) {
				return '';
			}
			if ( value.jquery ) {
				// Convert to an idrefs string
				return $.fn.idrefs ? $(value).idrefs() :
					Array.prototype.join.call(value.map(function() { return this.id; }), ' ');
			}
			if ( value.nodeType === 1 ) {
				// Assume this is a single DOM element, convert to an idref string
				return $.id ? $.id(value) : value.id;
			}
			if ( $.isArray(value) ) {
				// Assume this is tokens
				return value.join(' ');
			}
			// Otherwise just assume it is or can be converted to a string as normal
			return "" + value;
		},
	
	// Read an attribute and convert it to the specified datatype
	attr:
		function( elem, name, datatype ) {
			return this[datatype]( $.attr( elem, name ) );
		}
};

// Add functions to $.dt and $.xsd
$.each(['dt','xsd'], function() {
	if ( $[this] ) {
		$.extend($[this], fns);
	}
});

})(jQuery);

/*
 * jQuery Datatypes - Attribute functions 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
/* Depends:
 *   datatypes.core.js
 */
(function($) {

$.fn.extend({

// Toggle a boolean attribute state.
// Only toggles if the state is already set, unless a default value is given.
attrToggle: function( attr, def ) {
	return this.each(function() {
		var state = $.dt.bool( $.attr(this, attr) );

		// Use the given default as the current state if not set yet
		if ( state === undefined ) { state = def; }

		// Only toggle the state if it is already set
		if ( state !== undefined ) {
			$.attr(this, attr, ""+!state);
		}
	});
}

});

})(jQuery);

/*
 * jQuery Datatypes - Identifier functions 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
(function($) {

var prefix = 'id' + (+new Date()) + '-', count = 0;

// Get the id of an element, or generate a unique id if it lacks one
$.id = function( elem ) {
	if ( !elem.id ) {
		elem.id = prefix+(++count);
	}
	return elem.id;
};

// Returns a space separated list of the id's of the elements
$.fn.idrefs = function() {
	return Array.prototype.join.call($(this).map(function() { return $.id(this); }), ' ');
};

})(jQuery);

