/*! Build-0.3: mutations.core.js mutations.attr.js mutations.data.js mutations.html.js mutations.val.js */
/*!
 * jQuery Mutation Events 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
(jQuery.mutations || (function($) {

var m;
m = $.mutations = {

	REMOVAL:		3,
	ADDITION:		2,
	MODIFICATION:	1,
	INIT:			-1,
	
	// pre-mutation event prefix
	pre: 'pre-',

	// Construct a new mutation event
	event: function( type, params ) {
		var event = $.extend(new $.Event( type ), params);
		
		if ( event.attrChange === undefined ) {
			event.attrChange =
				event.newValue === null || event.newValue === undefined   ? m.REMOVAL :
				event.prevValue === null || event.prevValue === undefined ? m.ADDITION :
																			m.MODIFICATION;
		}
		
		return event;
	},
	
	// Trigger a pre and post mutation event, the pre mutation handlers may
	// cancel the mutation using event.preventDefault(), it may also modify
	// the mutation by setting the event fields.
	trigger: function( elem, eventType, eventParams, commit ) {
		var event = m.event( m.pre + eventType, eventParams ),
			opts = m.type[eventType],
			ret;
		
		if ( opts.pre ) {
//			console.log('trigger %s %s: %o -> %o %o', event.type, event.attrName, event.prevValue, event.newValue, elem);
			$.event.trigger( event, undefined, elem );
		} else {
			event.target = elem;
		}
		
		if ( !event.isDefaultPrevented() ) {
			event.type = eventType;
			ret = commit(event);
			
			if ( opts.post ) {
//				console.log('trigger %s %s: %o -> %o %o', event.type, event.attrName, event.prevValue, event.newValue, event.target);
				$.event.trigger( event, undefined, event.target );
			}
		}
		
		return ret;
	},

	type: {},
	
	// Register a new mutation event
	register: function( opts ) {
		m.type[opts.type] = opts;
		
		// Track how many bindings we have for this event type
		opts.pre = 0;
		opts.post = 0;

		// Register the pre/post mutation event types as special event type
		// so we can hook into jQuery on the first binding of this type		
		
		function special( eventType, stage ) {
			$.event.special[eventType] = {
				add: function(handler, data, namespaces) {
					
					// Call the setup on the first binding
					if ( !(opts.pre + opts.post) ) {
						opts.setup();
					}
					opts[stage]++;
					
					// If any namespaces are given prefixed with @ then limit
					// the handler to the bound element and attrNames specified
					// by the @-prefixed names.
					if ( namespaces.length ) {
						var attrNames = {}, proxy;
						
						$.each(namespaces, function() {
							if ( '@' === this.charAt(0) ) {
								attrNames[this.substring(1)] = true;
								proxy = true;
							}
						});
						
						if ( proxy ) {
							proxy = function(event) {
								if ( this === event.target && attrNames[event.attrName] ) {
									return handler.apply(this, arguments);
								}
							};
							proxy.type = handler.type;
							return proxy;
						}
					}
				},
		
				remove: function() {
					// Call teardown when last binding is removed
					opts[stage]--;
					if ( !(opts.pre + opts.post) ) {
						opts.teardown();
					}
				}
			};
		}
		
		special(m.pre + opts.type, 'pre');
		special(opts.type, 'post');
	}
};

$.fn.extend({
	// Trigger a fake mutation event for initialisation
	initMutation: function( type, names, defaultValue ) {
		var self = this, opts = m.type[type];
		
		if ( opts && opts.init && opts.post ) {
			if ( names === undefined ) {
				self.each(function() { opts.init(this, undefined, defaultValue); });
			} else {
				$.each(names.split(/\s+/), function(n, name) {
					self.each(function() { opts.init(this, name, defaultValue); });
				});
			}
		}
		
		return this;
	}
});

})(jQuery));

/*
 * jQuery.attr Mutation Events 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
(jQuery.mutations && (function($) {

$.mutations.register({
	// The event type triggered after a mutation,
	// "pre-" is prepended to this for the pre-mutation event.
	type: 'attr',
	
	// The blacklist can hold attributes that should never trigger an event
	blacklist: {},
	
	// Hook into jQuery when an event of this type is first bound
	setup: function() {
		var opts = this,
			attr = $.attr,
			trigger = $.mutations.trigger;
		
		// Save the original $.attr function
		this._attr = attr;
		
		// Override $.attr
		$.attr = function( elem, name, newValue, silent ) {
			var prevValue = attr(elem, name);

			if ( newValue === undefined ) {
				return prevValue;
			}

			if ( silent || opts.blacklist[name] ) {

				return attr(elem, name, newValue);
				
			} else if ( ""+newValue !== ""+prevValue ) { // Ensure a string comparison
				
				return trigger( elem, opts.type, {
						attrName: name,
						prevValue: prevValue,
						newValue: newValue === "" ? undefined : newValue
					},
					function( event ) {
						return attr( event.target, event.attrName, event.newValue );
					}
				);
			}
		};
	},
	
	// Remove hooks once all events of this type have been unbound
	teardown: function() {
		$.attr = this._attr;
		delete this._attr;
	},
	
	// Force an event to be trigger - useful for initialisation
	init: function( elem, name, defaultValue ) {
		var value = this._attr(elem, name) || defaultValue;
		if ( value !== undefined ) {
			$.event.trigger(
				$.mutations.event(this.type, {
					attrName: name,
					newValue: value,
					attrChange: $.mutations.INIT
				}),
				undefined, elem
			);
		}	
	}
});

})(jQuery));

/*
 * jQuery.data Mutation Events 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
(jQuery.mutations && (function($) {

$.mutations.register({
	// The event type triggered after a mutation,
	// "pre-" is prepended to this for the pre-mutation event.
	type: 'data',
	
	// The blacklist holds data item names that should never trigger an event
	blacklist: { 'events':true, 'handle':true, 'olddisplay':true, 'closest':true },
	
	// Hook into jQuery when an event of this type is first bound
	setup: function() {
		var opts = this,
			data = $.data,
			removeData = $.removeData,
			trigger = $.mutations.trigger;
		
		// Save the original $.data function
		this._data = data;
		this._removeData = removeData;
		
		// Override $.data
		$.data = function( elem, name, newValue, silent ) {
			var prevValue = data(elem, name);

			if ( newValue === undefined ) {
				return prevValue;
			}

			if ( silent || opts.blacklist[name] ) {

				return data(elem, name, newValue);
				
			} else if ( newValue !== prevValue ) {
				
				return trigger( elem, opts.type, {
						attrName: name,
						prevValue: prevValue,
						newValue: newValue
					},
					function( event ) {
						return data( event.target, event.attrName, event.newValue );
					}
				);
			}
		};
		
		$.removeData = function( elem, name, silent ) {
			if ( silent || opts.blacklist[name] ) {
				return removeData( elem, name );
			}

			return trigger( elem, opts.type, {
					attrName: name,
					prevValue: data(elem, name),
					attrChange: $.mutations.REMOVAL
				},
				function(event) {
					if ( event.attrChange === $.mutations.REMOVAL ) {
						removeData( event.target, event.attrName );
					} else {
						// The event handler wants the data modified rather than removed
						data( event.target, event.attrName, event.newValue );
					}
				}
			);
		};
	},
	
	// Remove hooks once all events of this type have been unbound
	teardown: function() {
		$.data = this._data;
		$.removeData = this._removeData;
		delete this._data;
		delete this._removeData;
	},
	
	// Force an event to be trigger - useful for initialisation
	init: function( elem, name, defaultValue ) {
		var value = this._data(elem, name) || defaultValue;
		if ( value !== undefined ) {
			$.event.trigger(
				$.mutations.event(this.type, {
					attrName: name,
					newValue: value,
					attrChange: $.mutations.INIT
				}),
				undefined, elem
			);
		}
	}
});

})(jQuery));

/*
 * jQuery.fn.html Mutation Events 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
(jQuery.mutations && (function($) {

$.mutations.register({
	type: 'html',
	
	setup: function() {
		var html = $.fn.html,
			trigger = $.mutations.trigger;
		
		this._html = html;
		
		$.fn.html = function( newValue, silent ) {
			if ( newValue === undefined ) {
				return html.apply(this);
			}
			
			if ( silent ) {
				return html.call( this, newValue );
			}
			
			return this.each(function() {
				trigger( this, 'html',
					{ newValue: newValue, attrChange: $.mutations.MODIFICATION },
					function( event ) {
						html.call( $(event.target), event.newValue );
					}
				);
			});
		};
	},
	
	teardown: function() {
		$.fn.html = this._html;
		delete this._html;
	},
	
	init: function( elem ) {
		$.event.trigger($.mutations.event(this.type, { attrChange: $.mutations.INIT }), undefined, elem);	
	}
});

})(jQuery));

/*
 * jQuery.fn.val Mutation Events 0.3
 *
 * Copyright (c) 2009 Adaptavist.com Ltd
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
(jQuery.mutations && (function($) {

$.mutations.register({
	type: 'val',
	
	setup: function() {
		var opts = this,
			val = $.fn.val,
			trigger = $.mutations.trigger;
		
		this._val = val;
		
		$.fn.val = function( newValue, silent ) {
			if ( newValue === undefined ) {
				return val.apply(this);
			}
			
			if ( silent ) {
				return val.call( this, newValue );
			}
			
			return this.each(function() {
				var prevValue = val.apply([this]);
				
				if ( newValue !== prevValue ) {
					trigger( this, opts.type,
						{ newValue: newValue, prevValue: prevValue},
						function( event ) {
							val.call( $(event.target), event.newValue );
						}
					);
				}
			});
		};
	},
	
	// Remove hooks once all events of this type have been unbound
	teardown: function() {
		$.fn.val = this._val;
		delete this._val;
	},
	
	init: function( elem, name ) {
		$.event.trigger($.mutations.event(this.type, { attrChange: $.mutations.INIT }), undefined, elem);	
	}
});

})(jQuery));

