/*jslint bitwise: true, eqeqeq: true, nomen: true, undef: true */
/*global jQuery, dwr */

/**
 * jquery.mvc-dwr.js
 * Load MVC data using DWR (direct-web-remoting)
 *
 * @author Mark Gibson
 * @version 0.3
 */
(function($) {

var bs = {},
	id = 1;

function batch() {
	return bs[id] || (bs[id] = { id: id, count: 0 });
}
function expire(b) {
	delete bs[b.id];
}

$.extend($.mvc.trail, {

	// Should existing data be overwritten?
	// The replace status is reset with each new batch
	dwrReplace: function(r) {
		batch().replace = (r === undefined) ? true : r;
		return this;
	},

	dwr: function(dwrFn) {
		var args = Array.prototype.slice.call(arguments, 1),
			path = $.mvc(this),
			b;
		
		// Don't overwrite existing data if replace is false
		if (!(bs[id] && bs[id].replace) && path.val() !== undefined) {
			return this;
		}
		
		b = batch();
		if (!b.count) {
//			console.log('mvc-dwr begin ['+b.id+']');
			dwr.engine.beginBatch();
		}
		
		// Add a callback function to the arguments
		args.push(function(data) {
			path.val(data);
			b.count--;
			if (!b.count && $.isFunction(b.complete)) {
//				console.log('mvc-dwr complete ['+b.id+']');
				b.complete.apply(b.that || path);
				expire(b);
			}
		});
		
		b.count++;
//		console.log('mvc-dwr call #'+b.count+' ['+b.id+']');		
		dwrFn.apply(this, args);
		
		return this;
	},
	
	// Save an object or the current path to pass to the callback as 'this'
	dwrThis: function(arg) {
		batch().that = arg || $.mvc(this);
		return this;
	},
	
	dwrGo: function(callback, opts) {
		var b = batch();
		id++;				// Any further .dwr calls start a new batch
		
		if (b.count) {
//			console.log('mvc-dwr end ['+b.id+']');
			b.complete = callback;
			dwr.engine.endBatch(opts);
		} else {
//			console.log('mvc-dwr callback ['+b.id+']');
			callback.apply(b.that || $.mvc(this));
			expire(b);
		}
		return this;
	}
});

})(jQuery);
