/*! Build-0.14: roles.aria.ui.js roles.button.ui.js roles.tablist.ui.js roles.tree.ui.js roles.dialog.ui.js */
/*
 * jQuery Roles - ARIA to jQuery UI styling 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* Apply jQuery UI CSS Framework
 *
 * Depends:
 *   roles.core.js
 *   datatypes.core.js
 */
(function($) {

$(':role')
	.roleStage('style', function() {
		$(this)
			.bind('attr.@aria-hidden', function(event) {
				$(this).toggleClass('ui-helper-hidden', $.dt.bool(event.newValue));
			});
	})
	
	.roleStage('init', function() {
		$(this)
			.initMutation('attr', 'aria-hidden', false);
	});

})(jQuery);

/*
 * jQuery Roles - Dialog implemented with jQuery UI 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* Depends:
 *   roles.core.js
 */
(function($) {

$(':role(button)')
	
	.roleStage('style', function() {
		$(this)
			.addClass('ui-button ui-widget ui-state-default ui-corner-all')
			
			.bind('attr.@disabled.@aria-disabled', function(event) {
				$(this).toggleClass('ui-state-disabled', $.roles.disabled(this));
			})
			
			.bind('focus.role-button', function() {
				$(this).addClass('ui-state-focus');
			})
			
			.bind('blur.role-button', function() {
				$(this).removeClass('ui-state-focus');
			})
			
			.bind('mouseenter.role-button', function() {
				if ( $.roles.disabled(this) ) { return; }
				$(this).addClass('ui-state-hover');
			})
			.bind('mouseleave.role-button', function() {
				$(this).removeClass('ui-state-hover');
			})
			
			.filter(':not([aria-pressed])')
				.bind('mousedown.role-button', function() {
					if ( $.roles.disabled(this) ) { return; }
					$(this).addClass('ui-state-active');
				})
				.bind('mouseup.role-button mouseleave.role-button', function() {
					$(this).removeClass('ui-state-active');
				})
			.end()
			
			.filter('[aria-pressed]')
				.bind('attr.@aria-pressed.role-button', function(event) {
					var pressed = $.dt.bool(event.newValue);
					$(this).removeClass('ui-state-highlight ui-state-error');
					
					if ( pressed === true ) {
						$(this).addClass('ui-state-highlight');
					} else if ( pressed !== false ) {
						$(this).addClass('ui-state-error');
					}
				})
			.end();
	});

})(jQuery);

/*
 * jQuery Roles - Tablist Styling 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* Apply jQuery UI CSS Framework
 *
 * Depends:
 *   roles.core.js
 *   datatypes.core.js
 */
(function($) {

$(':role(tablist)')
	.roleStage('style', function() {
		$(this)
			.addClass('ui-tabs-nav ui-helper-reset ui-helper-clearfix')
			
			.bind('attr.@aria-activedescendant', function(event) {
				$.dt.idrefs(event.prevValue).removeClass('ui-state-active');
				$.dt.idrefs(event.newValue).addClass('ui-state-active');
			});
	});

$(':role(tab)')
	.roleStage('dom', function() {
		// Ensure that tab content is wrapped in an <a>
		$(this)
			.filter('li:not(:has(a))')
			.wrapInner('<a href="#" tabindex="-1"></a>')
			.find('a')
			.bind('click.role-tab', function(event) { event.preventDefault(); });
	})
	
	.roleStage('style', function() {
		$(this)
			.addClass('ui-widget ui-state-default ui-corner-top')

			.bind('focus.role-tab', function() {
				$(this).addClass('ui-state-focus');
			})
			
			.bind('blur.role-tab', function() {
				$(this).removeClass('ui-state-focus');
			});
	});

})(jQuery);

/*
 * jQuery Roles - Tree Styling 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* Apply jQuery UI CSS Framework
 *
 * Depends:
 *   roles.core.js
 *   datatypes.core.js
 *   datatypes.attr.js
 */
(function($) {

// TODO: Allow this to be configured somewhere, eg:
var icons = [ 'ui-tree-toggle-none', 'ui-icon-triangle-1-e', 'ui-icon-triangle-1-s' ]; 

$(':role(tree)')
	.roleStage('style', function() {
		$(this)
			.addClass('ui-tree ui-widget ui-helper-reset')
			
			.bind('attr.@aria-activedescendant.role-tree', function(event) {
				$.dt.idrefs(event.prevValue).removeClass('ui-state-highlight');
				$.dt.idrefs(event.newValue).addClass('ui-state-highlight');
			})
			
			// Toggle expanded state of an iten when it's expander is clicked
			.bind('click.role-tree', function(event) {
				$(event.target).closest('.ui-tree-toggle').closest(':role(treeitem)').attrToggle('aria-expanded');
			})
			
			// Set the toggle icon according to the expanded state of a treeitem
			.bind('attr.aria-expanded.role-tree', function(event) {
				if ( event.attrName === 'aria-expanded' ) {
					var state = $.dt.bool(event.newValue),
						toggle = $(event.target).closest(':role(treeitem)').find('.ui-tree-toggle:first');
				
					toggle.removeClass(icons.join(' '));
				
					toggle.addClass(icons[state === undefined ? 0 : (state ? 2 : 1)]);
				}
			});
	});

$(':role(treeitem)')
	.roleStage('dom', function() {
		// Add a toggle widget to each treeitem
		$('<div class="ui-icon ui-tree-toggle" role="presentation"></div>')
			.addClass(icons[0])
			.prependTo(this);
	})
	
	.roleStage('style', function() {
		$(this)
			.addClass('ui-tree-item')
			
			.each(function() {
				// Find the group belonging to this item
				$(':role(group), + :role(group)', this).eq(0)
					.addClass('ui-tree-group ui-helper-reset');
			});
	});

})(jQuery);

/*
 * jQuery Roles - Dialog implemented with jQuery UI 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* Depends:
 *   roles.core.js
 *   mutations.core.js
 *   mutations.attr.js
 *   jquery.param.js
 *   ui.core.js
 *   ui.draggable.js
 *   ui.resizable.js
 */
(function($) {

function position( elem ) {
	var that = $(elem),
		elemH = that.outerHeight(true),
		elemW = that.outerWidth(true),
		winH = $(window).height(),
		winW = $(window).width(),
		top = $(document).scrollTop();
	
	if ( elemW > winW ) { elemW = that.width(winW - 20).outerWidth(true); }
	if ( elemH > winH ) { elemH = that.height(winH - 20).outerHeight(true); }
	
	that.css({
		'left': (winW - elemW)/2 + $(document).scrollLeft(),
		'top': Math.max(top, (winH - elemH)/2 + top)
	});
}

// role: dialog -> window
// attrs:
//  aria-hidden

$(':role(dialog)')

	.roleStage('bind', function() {
		$(this)
			.param('role', 'dialog')
			
			.roleAction('action-opened', function(event) {
				$(event.target)
					.prepend('<div class="ui-widget-shadow ui-widget-overlay ui-corner-all"></div>')
					.draggable({
						handle: ':role(heading)',
						distance: 10
					})
					.resizable({
						handles: 'all'
					})
					.css('position', 'absolute')
					.trigger('layout');
				
				position(event.target);
			})
			
			.roleAction('action-closed', function(event) {
				$(event.target)
					.resizable('destroy')
					.draggable('destroy')
					.find('.ui-widget-shadow:first').remove();
			});
	})
	
	.roleStage('style', function() {
		$(this)
			.addClass('ui-widget-content ui-corner-all');
	});

})(jQuery);

