/*! Build-0.14: jquery.within.js jquery.fixfocus.js jquery.param.js roles.core.js roles.aria.js roles.composite.js roles.option.js roles.button-toggle.js roles.button.js roles.toolbar.js roles.tablist.js roles.tree.js roles.listbox.js roles.dialog.js roles.layout.js roles.scrollable.js */
/*!
 * jQuery Within 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 */
/* Author: Mark Gibson (jollytoad at gmail dot com)
 *
 * Provides jQuery functions to find the next/previous element within the document,
 * and to filter a set of elements on whether they are descendants of another element.
 *
 * Very useful for keyboard navigation.
 */
(function($) {

// Check given element for match, returns element or undefined
function self( jq, expr ) {
	return $(jq).filter(expr || '*')[0];
}

// Search descendants
function descendants( jq, expr, prev ) {
	var found = $(jq).find(expr || '*');
	return found[prev ? found.length-1 : 0];
}

// Search following siblings
function nextSiblings( jq, expr ) {
	var found;
	$(jq).nextAll().each(function() {
		// Check the sibling and then it's descendants
		found = self(this, expr) || descendants(this, expr);
		return !found;
	});
	return found;
}

// Search preceding siblings
function prevSiblings( jq, expr ) {
	var found;
	$(jq).prevAll().each(function() {
		// Check the siblings descendants, then the sibling itself
		found = descendants(this, expr, true) || self(this, expr);
		return !found;
	});	
	return found;
}

// Search parents and preceding elements
function preceding( jq, expr ) {
	var found;
	$(jq).parents().each(function() {
		// Check the parent and then it's preceding siblings
		found = self(this, expr) || prevSiblings(this, expr);
		return !found;
	});
	return found;
}

// Find the next matching element within the document
$.fn.nextInDoc = function( expr ) {
	return this.pushStack(descendants(this, expr) || nextSiblings(this, expr) || nextSiblings($(this).parents(), expr),
			'nextInDoc', expr);
};

// Find the previous matching element within the document
$.fn.prevInDoc = function( expr ) {
	return this.pushStack(prevSiblings(this, expr) || preceding(this, expr),
			'prevInDoc', expr);
};

// Return just the elements that are descendants of the given element or the current context
$.fn.within = function( elem ) {
	var context = elem || this.context;
	return this.filter(function() {
		var cur = this;
		while ( cur ) {
			if ( cur == context ) {
				return true;
			}
			cur = cur.parentNode;
		}
		return false;
	});
};

})(jQuery);

/*!
 * jQuery - fix fn.focus() and focus-broken browsers 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
/* Make the jQuery focus() perform an actual focus on the first element.
 * Also adds support for document.activeElement if not present, and supports
 * focusing of any element with a tabindex.
 *
 * Depends:
 *   keys.core.js
 *   ui.core.js - for :tabbable/:focusable selectors
 */
(function($) {

$.support.noActiveElement = !( 'activeElement' in document );

var origfocus = $.fn.focus,
	realfocus;

$.fn.focus = function() {
	if ( arguments.length ) {
		return origfocus.apply(this, arguments);
	}
	if ( this[0] ) {
		realfocus(this[0]);
	}
	return this;
};


if ( $.support.noActiveElement ) {
	var active = function( elem ) {
		$(document.activeElement).removeClass('focus');
		document.activeElement = elem;
		$(document.activeElement).addClass('focus');
	};
	
	active(document.body);
	
	if ( document.addEventListener ) {
		document.addEventListener('focus', function(event) {
			if ( event && event.target ) {
				active(event.target);
			}
		}, true);
	}

	realfocus = function( elem ) {
		if ( elem !== document.activeElement ) {
			$(document.activeElement).blur();
			active(elem);
			$(elem).trigger('focus');
		}
	};
	
	$(document)
		.bind('keydown.key:tab', function(event) {
			// Focus the next tabbable element
			return !$(document.activeElement).nextInDoc(':tabbable').focus().size();
		})
		.bind('keydown.key:shift-tab', function(event) {
			// Focus the previous tabbable element
			return !$(document.activeElement).prevInDoc(':tabbable').focus().size();
		})
		.bind('mousedown', function(event) {
			if ( !event.isDefaultPrevented() ) {
				$(event.target).closest(':focusable').focus();
			}
		})
		.bind('keydown keyup keypress', function(event) {
			if ( event.target === document.body && document.activeElement !== document.body ) {
				// Re-trigger the event on the activeElement
				$(document.activeElement).trigger(event);
			}
		});
} else {
	realfocus = function( elem ) {
		window.setTimeout(function() {
			try {
				elem.focus();
			} catch (e) {}
		}, 0);
	};
}

})(jQuery);

/*!
 * jQuery Parameters 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
/* Add parameters to a jQuery object which can be used by jQuery functions
 * as default values. The prevObject chain will be a searched when fetching a parameter.
 */
(function($) {

var _end = $.fn.end;

$.fn.extend({

// param(name) - get a parameter
// param(name, value) - set a parameter
// param(object) - set a bunch of parameters at once
param: function( name, value ) {
	var options = name;

	if ( typeof name === "string" ) {
		if ( arguments.length > 1 ) {
			options = {};
			options[ name ] = value;
		} else {
			if ( this.param && this.param[name] ) {
				// Return the parameter
				return this.param[name];
			} else if ( this.prevObject ) {
				// Traverse up the prevObject chain looking for a param
				return this.prevObject.param(name);
			}
			// Nothing found
			return;
		}
	}

	this.param = $.extend(this.param || {}, options);

	return this;
},

// Overriden to ensure that any params are removed.
end: function() {
	delete this.param;
	return _end.apply(this, arguments);
}

});

})(jQuery);

/*!
 * jQuery Roles Framework 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
/* A widget activation framework based on the role concept of HTML5/WAI-ARIA,
 * and DOM Mutation Events.
 *
 * Depends:
 *   jquery.param.js
 *   keys.core.js - only if roleKey() is used.
 */
(jQuery.roles || (function($) {

var rolesRE = null;

$.roles = {
	version: '0.14',

	// Get the roles of an element
	get: function( elem ) {
		return elem.getAttribute('role');
	},

	activeElement: function() {
		return $(document.activeElement).filter(':role');
	},

	stages: [
		'dom',		// Any necessary DOM modifications prior to binding
		'bind',		// Setup bindings for states changes, actions,
					// and user interaction events (mouse, keys, focus, etc)
		'style',	// Setup styling and bindings for changing style
		'init',		// Trigger state initialisations
		'activate'	// Activate selections, timers, animations etc.
	],
	
	// Stages to be ran in a window.setTimeout
	timeout: {
		'activate': 0
	},
	
	// Mapping of role -> array of descendant roles
	hierarchy: {},
		
	add: function( role, inherits ) {
		var i = inherits.length, h = $.roles.hierarchy, r;
		while ( i-- ) {
			r = inherits[i];
			if ( !h[r] ) {
				h[r] = [];
				rolesRE = null;
			}
			h[r].push(role);
		}
		return this;
	}
};

$.fn.extend({

// Apply the role
roleSetup: function( stages ) {
	var that = this;
	stages = stages || this.param('roleStages') || $.roles.stages;

	$.each(stages, function(i, stage) {
		$(document).triggerHandler('role-'+stage, [that]);
	});

	return this;
},

// Register a setup stage for the role
roleStage: function( stage, fn ) {
	var selector = this.selector,
		timeout = $.roles.timeout[stage],
		handler;
	
	function runStage(event, context) {
		$(selector, context).each(function() {

/*DEBUG*roleStage*
			console.log(stage, this);
*DEBUG*roleStage*/

			fn.call(this, event);
		});
	}
	
	if ( typeof timeout === 'number' ) {
		handler = function(event, elems) {
			window.setTimeout(function() {
				runStage(event, elems);
			}, timeout);
		};
	} else {
		handler = runStage;
	}
	
	$(document).bind('role-'+stage, handler);
	return this;
},

// Register a role action
roleAction: function( action ) {
	var role = this.param('role') || 'unknown',
		args = Array.prototype.slice.apply(arguments);
	args[0] = action + '.role-' + role;
	return $.fn.bind.apply(this, args);
},

// Register a key -> event mapping for the role
roleKey: function( combo, action, keyEvent ) {
	var role = this.param('role') || 'unknown',
		type = keyEvent || this.param('keyEvent') || 'keydown';

	function handler(event) {
		if ( !event.isDefaultPrevented() ) {
			$.roles.activeElement().trigger(action);
		}
		event.preventDefault();
	}

	return this.unbind(type+'.key:'+combo).bind(type+'.role-'+role+'.key:'+combo, handler);
},

roleBind: function( type, triggerEvent ) {
	var role = this.param('role') || 'unknown';

	function handler(event) {
		$(event.target).trigger(triggerEvent);
	}

	return this.bind(type+'.role-'+role, handler);
}

}); // $.fn.extend


function rolesMatch( roleExpr, roles ) {
	var expr = roleExpr, m;
	
	if ( !rolesRE ) {
		rolesRE = (function(h) {
			var a = [];
			$.each(h, function(r) { a.push(r); });
			return new RegExp("\\b(" + a.join('|') + ")\\b", 'g');
		})($.roles.hierarchy);
	}
	
	while ( (m = rolesRE.exec(roleExpr)) && m[1] ) {
		expr += '|' + $.roles.hierarchy[m[1]].join('|');
	}
	
	return new RegExp("\\b(" + expr + ")\\b").test(roles);
}

// :role() selector
$.extend($.expr[':'], {
	role: function( elem, i, match ) {
		var roles = $.roles.get(elem);
		return roles && ( !match[3] || rolesMatch(match[3], roles) );
	}
});

})(jQuery)
);

/*
 * jQuery Roles - ARIA Utilities 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 */
/* Useful functions to levelage the features of WAI-ARIA.
 *
 * Depends:
 *   roles.core.js
 *   datatypes.core.js
 *   datatypes.utils.js
 *   datatypes.idrefs.js
 */
(jQuery.roles && (function($) {

$.extend($.roles, {

	// Find the elements controlled by the given element
	controls: function( elem ) {
		return $.dt.attr(elem, 'aria-controls', 'idrefs');
	},

	// Find the elements labelledby the given element
	labelledby: function( elem ) {
		return $("[aria-labelledby~='"+$.id(elem)+"']");
	},

	// Find elements controlled or labelled by the given element
	slaves: function( elem ) {
		return $.dt.attr(elem, 'aria-controls', 'idrefs').add("[aria-labelledby~='"+$.id(elem)+"']");
	},

	// Opposite of slaves
	masters: function( elem ) {
		return $.dt.attr(elem, 'aria-labelledby', 'idrefs').add("[aria-controls~='"+$.id(elem)+"']");
	},
	
	// Check whether the element is disabled
	disabled: function( elem ) {
		return elem.disabled || $.dt.bool( $.attr(elem, 'aria-disabled') );
	},
	
	// Call a function only when the given element becomes visible
	whenVisible: function( elem, namespace, fn ) {
		if ( !fn ) {
			// Remove any waiting function
			$(elem)
				.closest('[aria-hidden=true]')
				.unbind('attr.@aria-hidden.'+namespace);
				
		} else if ( $(elem).is(':visible') ) {
			// Element is visible so call the function now
			fn.apply(elem);
			return true;
			
		} else {
			// Wait until the closest hidden ancestor becomes unhidden
			$(elem)
				.closest('[aria-hidden=true]')
				.unbind('attr.@aria-hidden.'+namespace)
				.bind('attr.@aria-hidden.'+namespace, function(event) {
					if ( $.dt.bool(event.newValue) === false ) {
						// Unbind this handler
						$(this).unbind('attr.@aria-hidden.'+namespace);
					
						// Try a whenVisible again
						window.setTimeout(function() {
							$.roles.whenVisible( elem, namespace, fn );
						}, 0);
					}
				});
		}
	}

});

})(jQuery)
);

/*
 * jQuery Roles - Common functionality for ARIA Composite Widgets 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA roles: composite
 *
 * Depends:
 *   roles.core.js
 *   roles.aria.js
 *   mutations.core.js
 *   mutations.attr.js
 *   datatypes.core.js
 *   jquery.param.js
 *   jquery.within.js
 */
(function($) {

function selector( elem ) {
	return ($.data(elem, 'role-composite-descendants') || ':role(widget)');
}

// role: composite -> widget
// attrs: aria-activedescendant
// data: role-composite-descendants - a selector to restrict the composite items, defaults to ':role(widget)'

$(':role(composite)')

	.roleStage('bind', function() {
		$(this)
			.param('role', 'composite')
			
			// ---- ARIA States ----
			
			// Respond to the activedescendant being changed
			.bind('attr.@aria-activedescendant.role-composite', function(event) {
				// Deselect the previously activedescendant
				$.dt.idrefs(event.prevValue)
					.attr('aria-selected', false)
					.attr('tabindex', -1);

				// Select the new activedescendant
				$.dt.idrefs(event.newValue)
					.attr('aria-selected', true)
					.attr('tabindex', 0);
			})

			// ---- Actions ----

			// Focus previous descendant
			.roleAction('action-prev', function(event) {
				$(event.target).prevInDoc(selector(this)).within(this).focus();
				return false;
			})
			
			// Focus next descendant
			.roleAction('action-next', function(event) {
				$(event.target).nextInDoc(selector(this)).within(this).focus();
				return false;
			})
			
			// Focus first descendant
			.roleAction('action-first', function(event) {
				$(this).find(selector(this)+':first').focus();
				return false;
			})
			
			// Focus last descendant
			.roleAction('action-last', function(event) {
				$(this).find(selector(this)+':last').focus();
				return false;
			})
			
			// Focus the active descendant
			.roleAction('action-focus', function(event) {
				// Ensure that the item is focused
				var sel = selector(this),
					item = $(event.target).closest(sel).focus();
				
				// Focus the active item if clicked in a blank space
				if ( !item.length ) {
					item = $.dt.attr(this, 'aria-activedescendant', 'idrefs').focus();
			
					// Look for item with tabindex=0 if no activedescendant is set
					if ( !item.length ) {
						item = $(sel + '[tabindex=0]', this).focus();
					}
				}
			})
			
			.roleAction('set-activedescendant', function(event) {
				if ( $(event.target).is(selector(this)) ) {
					$.attr(this, 'aria-activedescendant', $.id(event.target));
					return false;
				}
			})

			.end();
	})

	.roleStage('activate', function() {
		$(this)
			// Activate the activedescendant
			.initMutation('attr', 'aria-activedescendant')
		
			// Select the first item if aria-activedescendant is not set
			.filter(':not([aria-activedescendant])')
				.find(selector(this)+':first')
					.trigger('set-activedescendant');
	});

})(jQuery);

/*
 * jQuery Roles - ARIA Option Widgets 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA role: option
 *
 * Depends:
 *   roles.core.js
 *   roles.composite.js
 *   mutations.core.js
 *   mutations.attr.js
 *   datatypes.core.js
 *   datatypes.attr.js
 *   jquery.param.js
 *   jquery.within.js
 */
(function($) {

// role: option -> input -> widget
// attrs:
//   tabindex - indicates active item

$(':role(option)')

	.roleStage('bind', function() {
		$(this)
			// Add a tabindex=-1 to allow click focus
			.attr('tabindex', -1)

			// ---- Focus ----

			// Set this item as the activedescendant of the tree
			.roleBind('focus', 'set-activedescendant');
	});

})(jQuery);

/*
 * jQuery Roles - Toggle Button 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA role: button (with aria-pressed set)
 *
 * Depends:
 *   roles.core.js
 *   mutations.core.js
 *   mutations.attr.js
 *   datatypes.core.js
 */
(function($) {

/* role: button -> input -> widget
 * attrs:
 *   aria-controls - the elements to trigger events or attribute changes
 *   aria-pressed  - if set, the targetted attribute will be set to the same state
 *   data-attr     - a boolean attribute to set on the target element
 */
$(':role(button)[aria-pressed][data-attr]')

	.roleStage('bind', function() {
		$(this)
			.bind('attr.@aria-pressed.role-button', function(event) {
				var attr = $.attr(this, 'data-attr'),
					pressed = $.dt.bool(event.newValue);
					
				if ( pressed === true || pressed === false ) {
					if ( /^\!/.test(attr) ) {
						attr = attr.substring(1);
						pressed = !pressed;
					}
			
					$.dt.attr(this, 'aria-controls', 'idrefs').attr(attr, pressed);
				}
			})
			
			.bind('attr.@aria-controls.role-button', function(event) {
				var button = this,
					id = $.id(this),
					attr = $.attr(button, 'data-attr'),
					not = /^\!/.test(attr);
				
				if ( not ) {
					attr = attr.substring(1);
				}
				
				// Unbind from previous target elements
				$.dt.idrefs(event.prevValue).unbind('.'+id);
				
				// Bind for changes to the new target attributes
				$.dt.idrefs(event.newValue).bind('attr.@'+attr+'.'+id, function(event) {
					
					// Use a timeout to ensure DOM is up to date
					window.setTimeout(function() {
						// Find the accumulative state of the attributes
						var state;					
					
						$.dt.attr(button, 'aria-controls', 'idrefs').each(function() {
							var val = $.dt.bool( $.attr(this, attr) );
						
							if ( val !== undefined ) {
								val = not ? !val : val;

								if ( state !== undefined && state !== val ) {
									state = 'mixed';
									return false;
								}
							
								state = val;
							}
						});
					
						if ( state !== undefined ) {
							$.attr(button, 'aria-pressed', state);
						}
					}, 0);
				});
			})
			
			.bind('click.role-button', function(event) {
				if ( $.roles.disabled(this) ) { return false; }
				$(this).attrToggle('aria-pressed');
				event.preventDefault();
			});
	})
	
	.roleStage('init', function() {
		$(this)
			.initMutation('attr', 'aria-controls aria-pressed');
	});

})(jQuery);

/*
 * jQuery Roles - ARIA Button Widget 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA role: button
 *
 * Depends:
 *   roles.core.js
 *   mutations.core.js
 *   mutations.attr.js
 *   datatypes.core.js
 */
(function($) {

$.roles.add('button', ['input','widget']);

function controls( button ) {
	var idrefs = $.attr(button, 'aria-controls');
	return idrefs ? $.dt.idrefs(idrefs) : $(button);
}

/* role: button -> input -> widget
 * attrs:
 *   aria-disabled
 *   aria-controls - the elements to trigger events or attribute changes (controls itself if not set)
 *   data-trigger  - an event to trigger on the target element
 *   data-attr     - an attribute to set/toggle on the target element
 *   data-val      - the value to set the attribute to, if not present, then the attribute is toggled
 */
$(':role(button)')
	
	.roleStage('bind', function() {
		$(this)
			.attr('tabindex', 0)
			
			// ---- Mouse ----
			
			.filter('[data-attr]:not([aria-pressed])')
				.bind('click.role-button', function(event) {
					if ( $.roles.disabled(this) ) { return false; }
					
					var val = $.attr(this, 'data-val'),
						attr =  $.attr(this, 'data-attr');
					
					if ( val === undefined ) {
						controls(this).attrToggle(attr);
					} else {
						controls(this).attr(attr, val);
					}
					event.preventDefault();
				})
			.end()
			
			.filter('[data-trigger]')
				.bind('click.role-button', function(event) {
					if ( $.roles.disabled(this) ) { return false; }
					controls(this).trigger( $.attr(this, 'data-trigger') );
					event.preventDefault();
				});
	});

})(jQuery);

/*
 * jQuery Roles - ARIA Toolbar Widget 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA roles: toolbar
 *
 * Depends:
 *   roles.core.js
 *   mutations.core.js
 *   mutations.attr.js
 *   jquery.within.js
 *   jquery.param.js
 */
(function($) {

// NOTE: This is not technically a 'composite' widget according to the spec,
// but it operates very much like one so we inherit it here.

$.roles.add('toolbar', ['composite']);


// role: toolbar -> group -> section -> structure

$(':role(toolbar)')

	.roleStage('bind', function() {
		$(this)
			.param('role', 'toolbar')
			
			.find(':role(widget)')
				.attr('tabindex', -1)
			.end()
			
			.find(':role(widget):first')
				.attr('tabindex', 0)
			.end()

			// ---- Mouse ----
			
			.roleBind('mouseup', 'action-focus')
			
			// ---- Keyboard ----
			
			.roleKey('left', 'action-prev')
			.roleKey('right', 'action-next')
			.roleKey('home', 'action-first')
			.roleKey('end', 'action-last')
					
			.end();
	});

})(jQuery);

/*
 * jQuery Roles - ARIA Tablist Widget 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA roles: tablist, tab, tabpanel
 *
 * Depends:
 *   roles.core.js
 *   roles.aria.js
 *   roles.composite.js
 *   mutations.core.js
 *   mutations.attr.js
 *   datatypes.core.js
 *   jquery.param.js
 *   jquery.within.js
 */
(function($) {

$.roles
	.add('tablist', ['composite','widget'])
	.add('tab', ['widget']);

// role: tablist +-> composite -> widget
//                \-> directory -> list -> region -> section -> structure

$(':role(tablist)')

	.roleStage('bind', function() {
		$(this)
			.data('role-composite-descendants', ':role(tab)')
			
			.param('role', 'tablist')

			// ---- Mouse ----
			
			.roleBind('mouseup', 'action-focus')
			
			// ---- Keyboard ----
			
			.roleKey('left', 'action-prev')
			.roleKey('right', 'action-next')
			.roleKey('up', 'action-prev')
			.roleKey('down', 'action-next')
			.roleKey('home', 'action-first')
			.roleKey('end', 'action-last')
			
			.end();
	});


// role: tab +-> sectionhead -> structure
//            \-> widget
// attrs: aria-selected

$(':role(tab)')

	.roleStage('bind', function() {
		function tabpanel( tab ) {
			return $.roles.slaves(tab).filter(':role(tabpanel)');
		}

		$(this)
			// Add a tabindex=-1 to allow click focus
			.attr('tabindex', '-1')
			
			// ---- ARIA States ----

			// Expand the associated 'tabpanel' when its tab is selected
			.bind('attr.@aria-selected.role-tab', function(event) {
				tabpanel(event.target).attr('aria-expanded', $.dt.bool(event.newValue));
			})
			
			// ---- Focus ----
			
			// Set this tab as the activedescendant of the tablist
			.roleBind('focus', 'set-activedescendant')
			.roleBind('focus', 'scroll-to');
	})

	.roleStage('init', function() {
		$(this)
			// Set the initial state of the tab and tabpanel
			.initMutation('attr', 'aria-selected', false);
	});


// role: tabpanel -> region -> section -> structure
// attrs: aria-expanded

$(':role(tabpanel)')

	.roleStage('bind', function() {
		function tab( tabpanel ) {
			return $.roles.masters(tabpanel).filter(':role(tab)');
		}
		
		$(this)
			.param('role', 'tabpanel')
			
			// ---- ARIA States ----
		
			// Show or hide the tabpanel if the expanded state is changed
			.bind('attr.@aria-expanded.role-tabpanel', function(event) {
				$.attr(this, 'aria-hidden', !$.dt.bool(event.newValue));
			})
			
			// ---- Actions ----
			
			// Focus the tab after our tab (ie. the tab for this tabpanel)
			.roleAction('action-next-tab', function() {
				tab(this).next(':role(tab)').focus();
				return false;
			})
			// Focus the tab preceding our tab
			.roleAction('action-prev-tab', function() {
				tab(this).prev(':role(tab)').focus();
				return false;
			})
			// Focus our tab
			.roleAction('action-focus-tab', function() {
				tab(this).focus();
				return false;
			})
			
			// ---- Keyboard ----
			
			.roleKey('ctrl-pageup', 'action-prev-tab')
			.roleKey('ctrl-pagedown', 'action-next-tab')
			.roleKey('ctrl-up', 'action-focus-tab')
			
			.end();
	});

})(jQuery);

/*
 * jQuery Roles - ARIA Tree Widget 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA roles: tree, treeitem
 *
 * Depends:
 *   roles.core.js
 *   roles.composite.js
 *   roles.option.js
 *   mutations.core.js
 *   mutations.attr.js
 *   datatypes.core.js
 *   datatypes.attr.js
 *   jquery.param.js
 *   jquery.within.js
 */
(function($) {

$.roles
	.add('tree', ['select','input','composite','widget'])
	.add('treeitem', ['option','input','widget']);


// find the sub-group belonging to a treeitem
function group( treeitem ) {
	return $(':role(group), + :role(group)', treeitem).eq(0);
}

// find the parent treeitem
function parent( treeitem ) {
	var item = $(treeitem).closest(':role(group)').closest(':role(treeitem)');
	if ( !item.length ) {
		item = item.end().prev(':role(treeitem)');
	}
	return item;
}


// role: tree -> select +-> composite -> widget
//                       \-> input -> widget
//                        \-> group -> section -> structure

$(':role(tree)')

	// Bind state (attribute) changes
	.roleStage('bind', function() {
		$(this)
			.data('role-composite-descendants', ':role(treeitem):visible')
			
			.param('role', 'tree')
						
			// ---- Actions ----

			// Climb up the tree
			.roleAction('action-climb', function(event) {
				// Is active item expanded?
				if ( $.dt.attr(event.target, 'aria-expanded', 'bool') === true ) {
					// Collapse the node
					$.attr(event.target, 'aria-expanded', false);
				} else {
					// Move to parent item
					parent(event.target).focus();
				}
				return false;
			})

			// Drill-down through treeitems
			.roleAction('action-drill', function(event) {
				// Is active item expanded?
				if ( $.dt.attr(event.target, 'aria-expanded', 'bool') === false ) {
					// Expand the node
					$.attr(event.target, 'aria-expanded', true);
				} else {
					// Move to first child item
					group(event.target).find(':role(treeitem):first').focus();
				}
				return false;
			})

			// Toggle expanded state
			.roleAction('action-toggle', function(event) {
				$(event.target).attrToggle('aria-expanded');
				return false;
			})
			
			// ---- Mouse ----

			// Expand/collapse on a double click
			.roleBind('dblclick', 'action-toggle')
			
			// Ensure focus
			.roleBind('mouseup', 'action-focus')
			
			// ---- Keyboard ----

			.roleKey('down', 'action-next')
			.roleKey('up', 'action-prev')
			.roleKey('left', 'action-climb')
			.roleKey('right', 'action-drill')
			.roleKey('enter', 'action-toggle')
			.roleKey('home', 'action-first')
			.roleKey('end', 'action-last')

			.end();

		// TODO: If a character key is pressed look for the next item
		// starting with that character.
	});


// role: treeitem +-> option -> input -> widget
//                 \-> listitem -> section -> structure
// attrs:
//   tabindex - indicates active item
//   aria-expanded - show/hide the related group

$(':role(treeitem)')

	.roleStage('bind', function() {
		$(this)			
			// ---- ARIA States ----

			// Show/hide the related group if expanded state has changed
			.bind('attr.@aria-expanded.role-treeitem', function(event) {
				// Find the group belonging to this treeitem and show/hide it
				group(event.target).attr('aria-hidden', !$.dt.bool(event.newValue));
			});
	})

	.roleStage('init', function() {
		$(this)
			// Initialise the hidden state of the related group
			.initMutation('attr', 'aria-expanded');
	});

})(jQuery);

/*
 * jQuery Roles - ARIA Listbox Widget 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA role: listbox
 *
 * Depends:
 *   roles.core.js
 *   roles.aria.js
 *   roles.composite.js
 *   mutations.core.js
 *   mutations.attr.js
 *   datatypes.core.js
 *   jquery.param.js
 *   jquery.within.js
 */
(function($) {

$.roles
	.add('listbox', ['select','composite','widget']);

// roles: listbox +-> select +-> composite -> widget
//                 \          \-> input -> widget
//                  \          \-> group -> section -> structure
//                   \-> list -> region -> section -> structure

$(':role(listbox)')

	.roleStage('bind', function() {
		$(this)
			.data('role-composite-descendants', ':role(option)')
			
			.param('role', 'listbox')

			// ---- Mouse ----
			
			.roleBind('mouseup', 'action-focus')
			
			// ---- Keyboard ----
			
			.roleKey('left', 'action-prev')
			.roleKey('right', 'action-next')
			.roleKey('up', 'action-prev')
			.roleKey('down', 'action-next')
			.roleKey('home', 'action-first')
			.roleKey('end', 'action-last')
			
			.end();
	});

})(jQuery);

/*
 * jQuery Roles - ARIA Dialog Widget 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* An implementation of the ARIA role: dialog
 *
 * Depends:
 *   roles.core.js
 *   mutations.core.js
 *   mutations.attr.js
 *   datatypes.core.js
 *   jquery.param.js
 */
(function($) {

// role: dialog -> window
// attrs:
//  aria-hidden

$(':role(dialog)')

	.roleStage('bind', function() {
		$(this)
			.bind('attr.@aria-hidden.role-dialog', function(event) {
				$(this).trigger($.dt.bool(event.newValue) ? 'action-closed' : 'action-opened');
			})
			
			.param('role', 'dialog')
			.roleKey('esc', 'action-cancel')
			.end();
	});

})(jQuery);

/*
 * jQuery Roles - Layout manager 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* Depends:
 *   roles.core.js
 *   jquery.defer.js - optional
 */
(function($) {

// TODO: Allow options to be configured
var opts = {
	namespace: 'ui-layout',
	container: '.ui-layout',
	center: '.ui-layout-center',
	north: '.ui-layout-north',
	east: '.ui-layout-east',
	south: '.ui-layout-south',
	west: '.ui-layout-west',
	inner: '.ui-layout-inner',
	resizeEvent: 'resize',
	resizeDefer: 200
};

var layoutBound = false;

// Get dimensions of surounding panels for a layout
function dims( context ) {
	return {
		top: $(opts.north, context).outerHeight(true) || 0,
		right: $(opts.east, context).outerWidth(true) || 0,
		bottom: $(opts.south, context).outerHeight(true) || 0,
		left: $(opts.west, context).outerWidth(true) || 0
	};
}

// Fix layout for faulty browsers (IE6)
function fixLayout() {
	var context = this;
	window.setTimeout(function() {
		var d = dims(context),
			h = $(context).height() - d.top - d.bottom,
			w = $(context).width();

		// Vertical position/size of middle row of panels
		$(opts.west+','+opts.center+','+opts.east, context).css('top', d.top).height(h);

		// Set the width of the top/bottom panels
		$(opts.north+','+opts.south, context).width(w);

		// Horizontal position/size of center panel
		$(opts.center, context).css('left', d.left).width(w - d.left - d.right);

		// Set height of a fitted inner element
		$(opts.center+' '+opts.inner, context).height(h-2);
	}, 0);
}

function onResize(handler) {
	$(document).bind(opts.resizeEvent+'.'+opts.namespace, $.defer ? $.defer(opts.resizeDefer, handler) : handler);
}

// Fix faulty browsers (IE6)
function fix( fixFn, context ) {
	fixFn.apply(context);
	
	if ( opts.resizeEvent ) {
		onResize(function(event) {
			$(event.target).find(opts.container).andSelf().filter(opts.container).each(fixFn);
		});
	}
}

// Check that the layout has worked, and apply fixes if not
function check( context ) {
	window.setTimeout(function() {
		var h = $(context).outerHeight();
		
		$(opts.north+','+opts.center+','+opts.south, context)
			.filter(':visible')
			.each(function() {
				h -= $(this).outerHeight();
			});
		
		if ( h !== 0 ) {
/*DEBUG*layout*
			alert('fix layout');
*DEBUG*layout*/
			fix(fixLayout, context);
		} else {
			// Check whether fitted inner elements fit as expected
			$(opts.inner, context).each(function() {
				var self = $(this),
					parent = $(this).parent();
				
				function fixHeight() {
					window.setTimeout(function() {
						self.height(parent.height());
					}, 0);
				}
				
				if ( self.height() !== parent.height() ) {
/*DEBUG*layout*
					alert('fix inner');
*DEBUG*layout*/
					fixHeight();
					onResize(fixHeight);
				}
			});
		}
	}, 0);
}

// Layout the panels
function layout() {
	var context = this;
	window.setTimeout(function() {
		if ( $(context).is(':visible') ) {
			var d = dims(context);
			$(opts.center, context).css(d);
			$(opts.west+','+opts.east, context).css({top: d.top, bottom: d.bottom});
			
			if ( !$(context).data('ui-layout-fixed') ) {
				check(context);
				$(context).data('ui-layout-fixed', true);
			}
		}
	}, 0);
}

// Automatically apply the layout
$(opts.container)
	.roleStage('bind', function() {
		if ( !layoutBound ) {
			$(document).bind('layout.'+opts.namespace, function(event) {
				$(event.target)
					.find(opts.container)
					.each(layout);
			});
			layoutBound = true;
		}
	})
	.roleStage('activate', function() {
		layout.apply(this);
	});

})(jQuery);

/*
 * jQuery Roles - Scrollable Area 0.14
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
/* Allow an large area to be scrolled within a smaller viewport,
 * without scrollbars.
 *
 * Depends:
 *   roles.core.js
 *   jquery.defer.js - optional, but recommended
 */
(function($) {

// TODO: Allow options to be configured
var opts = {
	namespace: 'ui-scrollable',
	container: '.ui-scrollable',
	scrollEvent: 'scroll-to',
	resizeEvent: 'resize',
	resizeDefer: 500
};

function scroll( delta ) {
	var plane = $('> div:first', this),
		inner = $('> div:first', plane).css('display', 'inline-block'),
		overflow = $(this).innerWidth() - inner.outerWidth(true);
	
	this.scrollLeft = 0;
	inner.css('display', 'block');
	plane.css('left', overflow < 0 ? Math.round(Math.max(overflow, Math.min(0, plane.position().left + (delta || 0)))) : 0);
}

var resizeBound = false;
function resizeSetup() {
	if ( !resizeBound && opts.resizeEvent ) {
		var handler = function(event) {
			$(event.target).find(opts.container).andSelf().filter(opts.container).each(scroll);
		};
		if ( $.defer ) {
			handler = $.defer(opts.resizeDefer, handler);
		}
		$(document).bind(opts.resizeEvent+'.'+opts.namespace, handler);
		resizeBound = true;
	}
}

// TODO: Fade in scroll buttons when the mouse lingers around the far edges of the viewport
// - this should probably be implemented in a separate module.

$(opts.container)

	.roleStage('dom', function() {
		// Wrap the content with two div's, the first is the large virtual plane
		// that will be moved around, the second is use to measure the actual
		// size of the content to prevent scrolling too far.
		$(this)
			.css('overflow', 'hidden')
			.wrapInner('<div><div></div></div>')
			.find('> div:first')
				.css({
					'position': 'relative',
					'left': 0,
					'width': '999999px'
				});
	})

	.roleStage('bind', function() {
		$(this)
			
			// Scroll the target element into view
			.bind(opts.scrollEvent+'.'+opts.namespace, function(event) {
				var outer = $(this),
					target = $(event.target),
					outerOffset = outer.offset(),
					targetOffset = target.offset(),
					scrollOffset = outerOffset.left - targetOffset.left;
				
				// Is the target off to the left?
				if ( scrollOffset > 0 ) {
					scroll.call(this, scrollOffset);
					return;
				}
				
				// Find the right hand offset
				scrollOffset = (outerOffset.left + outer.innerWidth()) - (targetOffset.left + target.outerWidth(true));
				
				if ( scrollOffset < 0 ) {
					scroll.call(this, scrollOffset);
				}
			})
			
			// ---- Mouse ----

			.bind('DOMMouseScroll.'+opts.namespace+' mousewheel.'+opts.namespace, function(event) {
				scroll.call(this, event.wheelDelta ? event.wheelDelta/10 : event.detail * -4);
				return false;
			});
		
		resizeSetup();
	});

})(jQuery);

