/*! Build-0.1: ui.extras.js ui.surface.js ui.area.js ui.vertical.js ui.wheel.js ui.triangle.js */
/*
 * jQuery UI Extra core features 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 */
(function($) {

$.ui.extras = {
	
	option: function(key, value) {
		var options = key,
			self = this;

		if (typeof key == "string") {
			if (value === undefined) {
				return this._getData(key);
			}
			options = {};
			options[key] = value;
		}

		$.each(options, function(key, value) {
			self._setData(key, value);
		});
		
		this._updated(options);
	},
	
	_getData: function(key) {
		return this._get && this._get[key] ? this._get[key].call(this, this.options[key]) : this.options[key];
	},
	
	_setData: function(key, value) {
		if (this._set && this._set[key]) {
			this.options[key] = this._set[key].call(this, value, this.options[key]);
		} else {
			this.options[key] = value;

			if (key == 'disabled') {
				this.element.toggleClass(
						this.widgetBaseClass + '-disabled' + ' ' +
						this.namespace + '-state-disabled', value)
					.attr("aria-disabled", value);
			}
		}
	},
	
	_updated: function() {}
};

})(jQuery);

/*
 * jQuery UI Surface 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  ui.core.js
 */
(function($) {

$.ui.surface = $.extend({}, $.ui.extras, $.ui.mouse, {

	_init: function() {
		this.element
			.css('position', 'relative')
			.addClass('ui-surface ' + this.widgetBaseClass)
			.disableSelection();

		this._mouseInit();
		
		if ( this.options.reticle !== false ) {
			this.reticle = $(this.options.reticle, this.element);
			
			if ( !this.reticle.length ) {
				this.reticle = $('<div class="ui-reticle"></div>').prependTo(this.element);
			}
			
			this.reticle.css({
				'position': 'absolute',
				'left': 0,
				'top': 0
			});
		}
		
		this._updated();
	},
	
	offset: function() {
		return this.element.offset();
	},
	
	_updated: function() {
		if ( this.reticle && this.reticle.length ) {
			this.reticle.css(this._getPoint());
		}
	},

	_mouseCapture: function() {
		return true;
	},
	
	_mouseStart: function(event) {
		this._mouseDrag(event);
		return true;
	},
	
	_mouseDrag: function(event) {
		var offset = this.offset();
		this._setPoint(event.pageX - offset.left, event.pageY - offset.top);
		this._updated();
		this._trigger('drag', event, this._ui());
		return false;
	},
	
	_mouseStop: function(event) {
		this._trigger('point', event, this._ui());
		return false;
	}
});

})(jQuery);

/*
 * jQuery UI Area (2D) Surface Widget 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  ui.core.js
 *  ui.surface.js
 */
(function($) {

function restrict(v) { return Math.max(0, Math.min(1, v)); }

$.widget('ui.area', $.extend({}, $.ui.surface, {

	_set: {
		x: restrict,
		y: restrict
	},

	_getPoint: function() {
		var limits = this.limits();
		return {
			left: Math.round(this._getData('x') * limits.width),
			top: Math.round(this._getData('y') * limits.height)
		};
	},
	
	_setPoint: function( x, y ) {
		var limits = this.limits();
		this._setData('x', x / limits.width);
		this._setData('y', y / limits.height);
	},
	
	_ui: function(event) {
		return { x: this._getData('x'), y: this._getData('y') };
	},

	limits: function() {
		return {
			width: this.element.innerWidth() - 1,
			height: this.element.innerHeight() - 1
		};
	}
}));

$.extend($.ui.area, {
	version: "0.1",
	eventPrefix: "point",
	
	defaults: {
		reticle: '.ui-reticle',
		cancel: null,
		delay: 0,
		distance: 0,
		x: 0,
		y: 0
	}
});

})(jQuery);

/*
 * jQuery UI Vertical Surface Widget 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  ui.core.js
 *  ui.surface.js
 */
(function($) {

$.widget('ui.vertical', $.extend({}, $.ui.surface, {

	_set: {
		value: function( value ) {
			return Math.max(0, Math.min(1, value));
		}
	},

	_getPoint: function() {
		return {
			top: Math.round(this._getData('value') * this.limit())
		};
	},
	
	_setPoint: function( x, y ) {
		this._setData('value', y / this.limit());
	},
	
	_ui: function(event) {
		return { value: this._getData('value') };
	},
	
	limit: function() {
		return this.element.innerHeight() - 1;
	}
}));

$.extend($.ui.vertical, {
	version: "0.1",
	eventPrefix: "point",
	
	defaults: {
		reticle: '.ui-reticle',
		cancel: null,
		delay: 0,
		distance: 0,
		value: 0
	}
});

})(jQuery);

/*
 * jQuery UI Wheel Surface Widget 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  ui.core.js
 *  ui.surface.js
 */
(function($) {

$.widget('ui.wheel', $.extend({}, $.ui.surface, {
	
	_getPoint: function() {
		var angle = this.options.value * 2*Math.PI,
			centre = this.centre(),
			radius = this.radius();
		
		return {
			left: Math.round(centre.x + Math.sin(angle) * radius.x),
			top: Math.round(centre.y - Math.cos(angle) * radius.y)
		};
	},
	
	_setPoint: function( x, y ) {
		var centre = this.centre();
		this.options.value = (Math.PI - Math.atan2(x - centre.x, y - centre.y)) / (2*Math.PI);
	},
	
	_ui: function(event) {
		return { value: this.options.value };
	},

	centre: function() {
		return {
			x: this.element.innerWidth()/2,
			y: this.element.innerHeight()/2		
		};
	},
	
	radius: function() {
		return {
			x: this.element.width()/2,
			y: this.element.height()/2
		};
	}

}));

$.extend($.ui.wheel, {
	version: "0.1",
	eventPrefix: "point",
	
	defaults: {
		reticle: '> .ui-reticle',
		cancel: null,
		delay: 0,
		distance: 0,
		value: 0
	}
});

})(jQuery);

/*
 * jQuery UI Triangle Surface Widget 0.1
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  ui.core.js
 *  ui.surface.js
 */
(function($) {

function restrict(v) { return Math.max(0, Math.min(1, v)); }

$.widget('ui.triangle', $.extend({}, $.ui.surface, {

	_mouseCapture: function(event) {
		var offset = this.offset();
		return this._setPoint(event.pageX - offset.left, event.pageY - offset.top,
								event.target !== this.reticle[0]);
	},
	
	_mouseStart: function(event) {
		this._updated();
		this._trigger('drag', event, this._ui());
		return true;
	},

	_set: {
		v: restrict,
		s: restrict
	},

	_getPoint: function() {
		var limits = this.limits(),
			v = this._getData('v'),
			s = this._getData('s');
		return {
			left: Math.round( ( (s*v) + (1-v)/2 ) * limits.width ),
			top: Math.round(v * limits.height)
		};
	},
	
	_setPoint: function( x, y, strict ) {
		var limits = this.limits(),
			v = y / limits.height,
			s = ( (x / limits.width) - (1-v)/2 ) / v;
		
		if (strict && (v < 0 || v > 1 || s < 0 || s > 1)) { return false; }
		
		this._setData('v', v);
		this._setData('s', s);
		return true;
	},
	
	_ui: function(event) {
		return { v: this._getData('v'), s: this._getData('s') };
	},

	limits: function() {
		return {
			width: this.element.innerWidth() - 1,
			height: this.element.innerHeight() - 1
		};
	}

}));

$.extend($.ui.triangle, {
	version: "0.1",
	eventPrefix: "point",
	
	defaults: {
		reticle: '> .ui-reticle',
		cancel: null,
		delay: 0,
		distance: 0,
		v: 0,
		s: 0
	}
});

})(jQuery);

