/*jslint forin:true */
/*global jQuery, ManageLayoutsDWR */

jQuery(function($) {
	
	$.fn.atbPanelCSS = function(prop, val, palette, visibleTest) {
		var self = this;
		
		function css(prop, val) {
			// IE will throw a wobbly if you attempt to set an invalid css value
			// so we catch it and set it to empty instead.
			try {
				self.css(prop, val);
			} catch (e) {
				self.css(prop, "");
			}
		}
		
		if ( val === null || val === 'empty' ) {
			val = '';
		}
		
		// Ignore non-CSS props
		switch (prop) {
			case "defer":
			case "content":
			case "renderMode":
				return;
		}
		
		switch (prop) {
			case "backgroundImage":
				if ( val && /([^:]*:){0,1}[^\^]*^.*/.test(val) ) {
					ManageLayoutsDWR.getBackgroundImage(val, $.defer(0, function(bgImage) {
						if ( bgImage ) {
							css(prop, bgImage);
						}
					}));
				} else {
					css(prop, val || "");
				}
				break;

			case "backgroundColor":
			case "borderColor":
			case "color":
				css(prop, palette[val] || val || "");
				break;

			case "visible":
				if ( visibleTest ) {
					val = visibleTest(val);
				}
				self.toggleClass('hidden', !val);
				break;

			case "collapse":
				// NOTE: collapse should have no effect on the preview
//				self.toggleClass('hidden', val);
				break;

			default:
				css(prop, val);
		}
	};
	
	$('.layout-preview')
		.closest('.atb-layout-data')

		// Update the layout preview panels
		.bind('atb-panel-changed.layout-preview', function(event, panelId) {
			var palette = $(this).data('atb-palette') || {},
				layoutData = $(this).data('atb-layout'),
				panelData = layoutData && layoutData[panelId],
				visibleTest;
			
			if ( panelData ) {
				// Force the slider to be hidden with its sidebar
				if ( /Slider$/.test(panelId) ) {
					visibleTest = function( val ) {
						return layoutData[panelId.replace('Slider', 'Sidebar')].visible ? val : false;
					};
				}

				// Apply the CSS properties to the panel element
				$('[data-panel='+panelId+']', this).each(function() {
					var panel = $(this), prop;
					for ( prop in panelData ) {
						panel.atbPanelCSS(prop, panelData[prop], palette, visibleTest);
					}
				});
			}
		})

		// Update all panels when the layout changes
		.bind('atb-layout-updated.layout-preview', function(event) {
			$('[data-panel]', this).each(function() {
				var panelId = $.attr(this, 'data-panel');
				$(this).trigger('atb-panel-changed', [ panelId ]);
			});
		})
		
		.trigger('atb-layout-refresh.layout-preview');

});

