/*global jQuery */

(function($) {
	
	$.fn.atbField = function() {
		function accum() {
			var ref = this.attr('data-ref') || "";
			return ref.charAt(0) === "." ? accum.apply($(this).parent().closest('[data-ref]')) + ref : ref;
		}
		
		function split( str ) {
			var a = $.trim(str).split(/[,\s]+/);
			if ( a.length === 1 && a[0] === "" ) { return []; }
			return a;
		}

		var self = this,
			outer = $(this).closest('[data-ref]'),
			path = accum.apply(outer);

		return {
			outer: outer,

			path: path,

			hasPermissions: function(layoutData) {
				var perms = (outer.attr('data-perm') || "").split(/\s+/),
					i = perms.length;
				while (--i) {
					if ( !layoutData.hasPermission(perms[i]) ) { return false; }
				}
				return true;			
			},

			getDataPair: function(root) {
				var obj = root || this.getLayout(),
					keys = path.split('.'),
					prop = keys.shift();
			
				while ( obj && keys.length ) {
					obj = obj[prop];
					prop = keys.shift();
				}
				return { obj: obj, prop: prop };
			},

			getDataValue: function(root) {
				var pair = this.getDataPair(root);
				return pair.obj[pair.prop];
			},

			setDataValue: function(value, root) {
				var pair = this.getDataPair(root);
				if ( !$.isEqual(value, pair.obj[pair.prop]) ) {
					pair.obj[pair.prop] = value;
					return true;
				}
			},

			getViewValue: function() {
				if ( outer.attr('data-type') === 'array' ) {
					return split(self.val());
				}
				if ( self.is(':checkbox') ) {
					return self.is(':checked');
				}
				// If value was null, then an empty string is converted to null
				if ( self.attr('data-null') === 'true' && self.val() === "" ) {
					return null;
				}
				return self.val();
			},

			setViewValue: function(value) {
				if ( $.isFunction(value) ) {
					value = value();
				}
				if ( $.isArray(value) ) {
					value = value.join(', ');
				}
				if ( self.is(':checkbox') ) {
					self[0].checked = !!value;
				} else if ( self.is(':input') ) {
					self.val(value === undefined || value === null ? "" : value);
				} else {
					self.text(value === undefined || value === null ? "" : value);
				}
				// Note that value was null
				self.attr('data-null', value === null ? 'true' : 'false');
			},
			
			setViewOrigin: function(value) {
				outer.attr('data-origin', this.getOrigin(value) || 'unknown');
			},

			getOrigin: function(value, root) {
				var layer = root || this.getLayout(),
					origin;
				
				while ( layer ) {
					if ( $.isEqual(value, this.getDataValue(layer)) ) {
						// The value is set in this layer
						origin = layer.getLayerId();
					} else if ( origin ) {
						// The value was already set in a higher layer
						break;
					}
					layer = layer.getPrototype && layer.getPrototype();
				}
				return origin;
			},

			getLayer: function() {
				return $(self).closest('[data-layer]').attr('data-layer') || 'edits';
			},

			getLayout: function(layer) {
				return ($(self).closest('.atb-layout-data').data('atb-layout-layers') || {})[layer || this.getLayer()] || {};
			},
			
			trigger: function() {
				self.trigger('atb-field-changed', [ this ]);
			}
		};
	};

	// Setup a the field from the given layered data
	$.fn.atbSetupField = function() {
		var layoutData = $('#layout-manager').data('atb-layout'),
			noEdit = !(layoutData && layoutData.hasPermission('edit'));
		
		return this
			.unbind('.atb-field')
			
			// Setup inputs
			.each(function() {
				var field = $(this).atbField(),
					value = field.getDataValue(),
					disabled = $(this).hasClass('force-disabled');
				
				$.attr(this, 'disabled', noEdit || disabled || !field.hasPermissions(layoutData) || value === undefined);
				
				// Set the input value
				field.setViewValue(value);
				
				if ( value !== undefined ) {
					
					field.setViewOrigin(value);
					
					field.outer
						// Ensure the property is visible
						.removeClass('atb-field-undefined')
						.addClass('atb-field');
					
					$(this)
						.bind('change.atb-field, blur.atb-field', function() {
							var field = $(this).atbField(),
								value = field.getViewValue();

							if ( field.setDataValue(value) ) {
								field.setViewOrigin(value);
								field.trigger();
							}
						})
						.bind('atb-field-revert.atb-field', function(event, layer) {
							var field = $(this).atbField(),
								value = field.getDataValue(field.getLayout(layer));
							
							if ( field.setDataValue(value) ) {
								field.setViewValue(value);
								field.setViewOrigin(value);
								field.trigger();
							}
						})
						.filter(':text,textarea')
							.bind('keyup.atb-field', $.defer(666, function() {
								$(this).trigger('change');
							}));

				} else {
					field.outer
						.removeClass('atb-field')
						.addClass('atb-field-undefined')
						.removeAttr('data-origin');
				}
			})
			
			// Setup label associations
			.not('[id]')
			.each(function() {
				$(this)
					.closest('[data-ref]')
					.find('label:not([for])')
					.attr('for', $(this).idrefs());
			});
	};

})(jQuery);

