/*global jQuery, window, alert, confirm */

jQuery(function($) {

	var context = $('#tabpanel-resources'),
		table = $('table.resource-list', context),
		list = $('.resource-list-body', table),
		template = $('.resource-item.template', list),
		urlPrefix = $('#confluence-context-path').attr('content') + '/plugins/servlet/builder/resource/',
		uploadCount = 0,
		reEditable = /\.(css|htc|html?|js|wiki|xml)$/;
	
	function resourceURL( layoutId, resourceId ) {
		return urlPrefix + layoutId + '/' + resourceId + '?' + (new Date()).valueOf();
	}

	function resourceType( resourceData ) {
		if ( reEditable.test(resourceData.name) ) {
			return 'text';
		}
		return resourceData.contentType.split('/', 1)[0];
	}

	// Reload the layout to show the new resources	
	function reload(layoutId) {
		context.trigger('atb-layout-select', [ layoutId, { reload:true, keepEdits:true, noConfirm:true } ]);
	}

	function editResource( layoutData, resourceData, refresh ) {
		var dialog = $('#layout-manager-resource-editor'),
			textarea = $('#resource-editor-content'),
			actionURL = $('#resource-upload-form').attr('action');
		
		function commit( close ) {
			$.post(actionURL, {
				layoutId: layoutData.layoutId,
				filename: resourceData.name,
				data: textarea.val(),
				description: resourceData.description
			}, function() {
				dialog.removeClass('dialog-modified');
				if ( close ) {
					dialog.attr('aria-hidden', true);
				}
				if ( refresh ) {
					reload(layoutData.layoutId);
				}
			});
		}

		// Set resource name as tab text
		$('#tab-resource-content a').text(resourceData.name);

		// Load resource content
		$.get(resourceURL(layoutData.layoutId, resourceData.name), function(data) {
			var original = data;
			
			// Populate textarea with resource content
			textarea
				.val(data)
				.bind('change', function() {
					dialog.toggleClass('dialog-modified', $(this).val() !== original);
					return false;
				});

			// Initialise dialog actions
			$('#layout-manager-resource-editor')
				.unbind('.resource-editor')
				.bind('dialog-apply.resource-editor', function() {
					commit(false);
					return false;
				})
				.bind('dialog-ok.resource-editor', function() {
					commit(true);
					return false;
				})
				.bind('dialog-cancel.resource-editor', function() {
					textarea.val('');
					dialog
						.removeClass('dialog-modified')
						.attr('aria-hidden', true);
					return false;
				})
				.trigger('atb-activate');
		}, 'text');
	}
	
	function deleteResource( layoutData, resourceData ) {
		var actionURL = $('#resource-upload-form').attr('action').replace(/[a-z]+\.action/, 'removeresource.action');

		if ( confirm('Delete resource: ' + resourceData.name + '\nAre you sure?') ) {
			$.post(actionURL, {
				layoutId: layoutData.layoutId,
				filename: resourceData.name
			}, function() {
				// Delete resource from overlay & edits layers
				delete layoutData.getPrototype().resources[resourceData.name];
				delete layoutData.resources[resourceData.name];
				reload(layoutData.layoutId);
			});		
		}
	}

	function refreshResourceList() {
		var layoutData = $('#layout-manager').data('atb-layout'),
			canEdit = layoutData.hasPermission('edit');

		// Disable upload form if user hasn't got edit permissions
		$('#resource-upload-form input').attr('disabled', !canEdit);

		// Empty resource list
		$('.resource-item', list).not('.template').remove();
		
		$.each(layoutData.resources, function(key, resourceData) {
			
			// Skip non resource objects
			if ( !(typeof resourceData === 'object' && resourceData.name) ) {
				return;
			}

			// Is the resource owned by the current layout (rather than inherited)
			// (getPrototype is used to retrieve data from the 'overlay' layer)
			var owned = resourceData.getPrototype().hasOwnProperty('name');

			template
				.clone()
				.removeClass('template')
				.attr('data-origin', owned ? 'overlay' : 'parent')
				.find('[data-ref]')
					.each(function() {
						var ref = $.attr(this, 'data-ref');
						if ( resourceData[ref] ) {
							$(this).text(resourceData[ref]);
						}
					})
				.end()
				.appendTo(list)

				.find('td:not(.resource-actions)')
					.css('cursor', 'pointer')
					.bind('click', function() {
						$(this).trigger('atb-resource-select', [ layoutData.layoutId, resourceData ]);
					})
				.end()

				.find('a.resource-download')
					.attr('href', resourceURL(layoutData.layoutId, resourceData.name))
				.end()

				.find('a.resource-edit')
					.each(function() {
						// Content type is text or filename implies a text file
						if ( canEdit && resourceType(resourceData) === 'text' ) {
							$(this).bind('click', function(event) {
								editResource.call( this, layoutData, resourceData, !owned );
								event.preventDefault();
							});
						} else {
							$(this).css('visibility', 'hidden');
						}
					})
				.end()
				
				.find('a.resource-delete')
					.each(function() {
						if ( canEdit && owned ) {
							$(this).bind('click', function(event) {
								deleteResource.call( this, layoutData, resourceData );
								event.preventDefault();
							});
						} else {
							$(this).css('visibility', 'hidden');
						}
					})
				.end();
		});
		
		// Resort the list
		table.trigger('update');
	}
	
	function createPreview( layoutId, resourceData ) {
		var category = resourceData && resourceType(resourceData),
			ret = '';
		
		switch (category) {
		case 'image':
			ret = $('<img class="preview-image" />')
					.attr('alt', resourceData.name)
					.attr('title', resourceData.name)
					.attr('src', resourceURL(layoutId, resourceData.name));
			break;

		case 'text':
			ret = $('<pre class="preview-text"></pre>')
					.attr('title', resourceData.name);
			
			$.get(resourceURL(layoutId, resourceData.name), function(data) {
				// Ensure line endings are CR+LF for IE7!
				ret.text(data.replace(/\r?\n/g, "\r\n"));
			}, 'text');
			break;
		}
		
		return ret;
	}
	
	// Upload form
	
	$('form', context)

		.bind('submit.resource-tab', function(event) {
			var layoutId = $('#layout-manager').data('atb-layout').layoutId,
				form = this,
				filename = $(':input[type=file]', form).val();
			
			if ( filename ) {		
				// Set the layout id
				$(':input[name=layoutId]', form).val(layoutId);
				
				$('#resource-upload-iframe').one('load.resource-upload', function(event) {
					form.reset();
					reload(layoutId);
				});
				
				$(form).trigger('atb-loading');
			}
		});

	// Make the resource list sortable
	table
		.tablesorter({
			headers: { 2: { sorter: false } }
		})
		.addClass('tablesorter')
		.find('th.header')
			.append('<span class="ui-icon ui-icon-triangle-2-n-s sort-icon"/>')
		.end();

	$(context)

//		.unbind('.resource-tab')

		.bind('atb-resource-select.resource-tab', function(event, layoutId, resourceData) {
			// Update description textarea
			$('.resource-desc', this)
				.val(resourceData ? resourceData.description : '');
			
			// Update preview area
			$('.resource-preview', this)
				.empty()
				.append(createPreview(layoutId, resourceData));
			
			if ( resourceData ) {
				$('.resource-item', list)
					.each(function() {
						$(this).toggleClass('resource-current', $('td[data-ref=name]', this).text() === resourceData.name);
					});
			}
				
//			console.log(resourceData);
		})
		
		.closest('.atb-layout-data')

			.bind('atb-layout-updated.resource-tab', function(event) {
				context.trigger('atb-resource-select');
				refreshResourceList();
			})
			
			.trigger('atb-layout-refresh.resource-tab');
});

