/*jslint bitwise: true, eqeqeq: true, nomen: true, undef: true */
/*global jQuery, ThemeAdminDWR, Option, alert */

jQuery(function($) {

	var panel = $('#aliases-panel');
	
	function busy() { panel.busy(true); }
	function done() { panel.busy(false); }
	
	// TODO: Move this to somewhere reusable
	if (!$.fn.zebra) {
		$.fn.zebra = function() {
			return this
				.removeClass('rowAlternate')
				.filter(':not(.hidden):odd')
				.addClass('rowAlternate')
				.end();
		};
	}
	
	function locations() {
		var l = {};
		$.mvc('/aliases/plugins/*/modules/*/location').walk({
			target: function(parent) {
				var plugin = this.toStep(2).val();
				if (!l[parent.location]) {
					l[parent.location] = {};
				}
				l[parent.location][parent.key] = {
					plugin: plugin.key,
					module: parent.key,
					name: plugin.name + ' - ' + parent.name
				};
			}
		});
		return l;
	}
	
	// Custom rendering for the 'Target' column
	$.mvc.setter['alias-target'] = function(v) {
		$(this).empty();
		if (v.url) {
			$('<a target="_blank"/>').attr('href', v.url).text(v.url).appendTo(this);
		} else {
			$(this).text($.mvc('/aliases/locations').addStep(v.location, v.key, 'name').val());
		}
	};
	
	// Lower-case and remove spaces from alias name
	$.mvc.getter['alias-name'] = function() {
		return $(this).val().toLowerCase().replace(/\s+/g,'');
	};
	
	function options() {
		var select = this,
			// Get items and sort on the name
			items = $.mvc(select,'mvc-options').val().toArray()
				.sort(function(a,b) { return a.name.localeCompare(b.name); });

		// Remove existing options
		select.options.length = 0;

		// Create and add options
		$.each(items, function() {
			select.options[select.options.length] = new Option(this.name, this.key);
		});

		// Select the first option
		select.selectedIndex = 0;
	}
	
	function invalid(field) {
		alert($('#alias-invalid-'+field).text());
	}
	
	function setup() {
		
		// Build locations lookup
		$.mvc('/aliases/locations').val(locations());

		// Setup Plugin/Module SELECTs
		$('select#plugin', panel)
			.each(options)			// Add OPTIONs
			.mvc()
			.bind(function() {		// Rebuild module SELECT when plugin changes
				$('select#module', panel).each(options);
			});
		
		// Init model-view bindings
		$(panel).mvcInit();
		
		// Disable form controls for the non-selected alias type
		$.mvc('/aliases/edit/type').bind(function(ev) {
			var type = ev.value || this.val();
			$('.alias-controls', panel).each(function() {
				var that = $(this);
				that.find(':input').attr('disabled', !that.is('.alias-type-'+type));
			});
		})
		.trigger('init');
		
		var aliasAction = {
			edit: function(path) {
				// Copy alias data into the editing area
				var alias = path.val();
	
				var editPath = $.mvc('/aliases/edit/-')
					.key('alias').val(alias.alias)
					.key('rel').val(alias.rel);
	
				if (alias.url) {
					editPath
						.key('type').val('url')
						.key('url').val(alias.url);
				} else if (alias.location && alias.key) {
					var loc = $.mvc('/aliases/locations').addStep(alias.location, alias.key).val();
					editPath
						.key('type').val('module')
						.key('plugin').val(loc.plugin)
						.key('module').val(loc.module)
						.key('url').val(null);
				}
			},
			del: function(path) {
				// Delete the alias
				path.del();
			}
		};
		
		// Alias action buttons
		$('#aliases-table > tbody', panel)
			.bind('click', function(ev) {
				var action = $(ev.target).closest('a.action').attr('value');
				
				if (action && aliasAction[action]) {
					var path = $(ev.target).closest('tr').mvc('mvc-repeat');
					ev.preventDefault();
					aliasAction[action](path);
				}
			});
		
		// Save changes
		$('input[name=save]', panel).bind('click', function() {
			busy();
			
			var fail = false,
				edit = $.mvc('/aliases/edit').val(),
				alias = {
					alias: edit.alias,
					rel: edit.rel,
					url: null,
					location: null,
					key: null
				};
			
			// TODO: Validation should really be performed on /aliases/aliases/*/* data
			// although improvements to the binding system are required first.
			
			if (!alias.alias) {
				// TODO: Should this be limited to a certain set of characters?
				invalid('alias');
				fail = true;
			}
			// TODO: Should 'rel' be validated?
			
			if (edit.type === 'url') {
				if (!edit.url) {
					// TODO: Validate URL
					invalid('url');
					fail = true;
				}
				alias.url = edit.url;
			} else if (edit.type === 'module') {
				var module = $.mvc('/aliases/plugins').addStep(edit.plugin,'modules',edit.module).val();
				alias.location = module.location;
				alias.key = module.key;
			} else {
				fail = true;
			}
			
			if (!fail) {
				$.mvc('/aliases/aliases').addStep(alias.alias).val(alias);
			}
			
			done();
		});
		
		// Commit changes to the server
		$.mvc('/aliases/aliases')
			.bind({ type: 'add set', each: function(ev) {
			
				// Manually trigger an update of the properties of the aliases
				// TODO: This should eventually be performed by MVC val(...).
				if (ev.type === 'set') {
					var path = $.mvc('/aliases/aliases').addStep(ev.value.alias,'-');
					$.each(ev.value, function(k,v) {
						var w = ev.was[k];
						if (v !== w) {
							path.key(k);
							path.trigger({
								type:   ev.type,
								target: $.mvc(path),
								value:  v,
								was:    w,
								source: ev.source
							});
						}
					});
				}
				
				busy();
				ThemeAdminDWR.addAlias(ev.value, done);
			}})
			.bind({ type: 'delete', each: function(ev) {
				busy();
				ThemeAdminDWR.removeAlias(ev.was.alias, done);
			}});
		
		// Init tablesorter
		$('#aliases-table', panel)
			.tablesorter()
			.addClass('tablesorter')
			.find('> tbody > tr')
			.zebra();

		panel.trigger('ready');
	}
	
	$.mvc('/aliases/-')
		.key('plugins').dwr(ThemeAdminDWR.getPluginData)
		.key('aliases').dwr(ThemeAdminDWR.getAliases)
		.dwrGo(setup);
});

